@@ -111,7 +111,7 @@ void mexFunction(int POutputCount,  mxArray* POutput[], int PInputCount, const m
 	if(mxIsDouble(PInputs[0]) && !mxIsComplex(PInputs[0]) && !mxIsSparse(PInputs[0]) )
 		LInput  = mxGetPr(PInputs[0]);
 	else 	
-		mexErrMsgTxt("First argument must be and not sparse REAL/DOUBLE.");
+		mexErrMsgTxt("First argument must be double and not sparse REAL/DOUBLE.");
 
     	// get 2nd argument
     	if  (PInputCount > 1) {
