@@ -1,50 +1,60 @@
-/*
+/* 
+  This code was extracted from libsvm 3.2.3 in Feb 2019 and 
+  modified for the use with Octave and Matlab
 
-Copyright (c) 2000-2012 Chih-Chung Chang and Chih-Jen Lin
-Copyright (c) 2010,2011,2015,2016 Alois Schloegl
-This function is part of the NaN-toolbox
-http://pub.ist.ac.at/~schloegl/matlab/NaN/
 
-This code was extracted from libsvm-3.12 in Apr 2015 and 
-modified for the use with Octave 
+Copyright (c) 2000-2019 Chih-Chung Chang and Chih-Jen Lin
+All rights reserved.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 3 of the License, or
-(at your option) any later version.
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions
+are met:
 
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
+1. Redistributions of source code must retain the above copyright
+notice, this list of conditions and the following disclaimer.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, see <http://www.gnu.org/licenses/>.
+2. Redistributions in binary form must reproduce the above copyright
+notice, this list of conditions and the following disclaimer in the
+documentation and/or other materials provided with the distribution.
 
-*/
+3. Neither name of copyright holders nor the names of its contributors
+may be used to endorse or promote products derived from this software
+without specific prior written permission.
 
 
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR
+CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+*/ 
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(HAVE_EXTERNAL_LIBSVM)
-  #include <libsvm/svm.h>
-#else
-  #include "svm.h"
-#endif
+#include "svm.h"
 
 #include "mex.h"
 #include "svm_model_matlab.h"
 
-#ifdef tmwtypes_h
-  #if (MX_API_VER<=0x07020000)
-    typedef int mwSize;
-    typedef int mwIndex;
-  #endif 
-#endif 
+#ifdef MX_API_VER
+#if MX_API_VER < 0x07030000
+typedef int mwIndex;
+#endif
+#endif
 
 #define CMD_LEN 2048
 
+int print_null(const char *s,...) {return 0;}
+int (*info)(const char *fmt,...) = &mexPrintf;
+
 void read_sparse_instance(const mxArray *prhs, int index, struct svm_node *x)
 {
 	int i, j, low, high;
@@ -67,22 +77,23 @@ void read_sparse_instance(const mxArray *prhs, int index, struct svm_node *x)
 	x[j].index = -1;
 }
 
-static void fake_answer(mxArray *plhs[])
+static void fake_answer(int nlhs, mxArray *plhs[])
 {
-	plhs[0] = mxCreateDoubleMatrix(0, 0, mxREAL);
-	plhs[1] = mxCreateDoubleMatrix(0, 0, mxREAL);
-	plhs[2] = mxCreateDoubleMatrix(0, 0, mxREAL);
+	int i;
+	for(i=0;i<nlhs;i++)
+		plhs[i] = mxCreateDoubleMatrix(0, 0, mxREAL);
 }
 
-void predict(mxArray *plhs[], const mxArray *prhs[], struct svm_model *model, const int predict_probability)
+void predict(int nlhs, mxArray *plhs[], const mxArray *prhs[], struct svm_model *model, const int predict_probability)
 {
 	int label_vector_row_num, label_vector_col_num;
 	int feature_number, testing_instance_number;
 	int instance_index;
-	double *ptr_instance, *ptr_label, *ptr_predict_label; 
+	double *ptr_instance, *ptr_label, *ptr_predict_label;
 	double *ptr_prob_estimates, *ptr_dec_values, *ptr;
 	struct svm_node *x;
 	mxArray *pplhs[1]; // transposed instance sparse matrix
+	mxArray *tplhs[3]; // temporary storage for plhs[]
 
 	int correct = 0;
 	int total = 0;
@@ -102,13 +113,13 @@ void predict(mxArray *plhs[], const mxArray *prhs[], struct svm_model *model, co
 	if(label_vector_row_num!=testing_instance_number)
 	{
 		mexPrintf("Length of label vector does not match # of instances.\n");
-		fake_answer(plhs);
+		fake_answer(nlhs, plhs);
 		return;
 	}
 	if(label_vector_col_num!=1)
 	{
 		mexPrintf("label (1st argument) should be a vector (# of column is 1).\n");
-		fake_answer(plhs);
+		fake_answer(nlhs, plhs);
 		return;
 	}
 
@@ -126,7 +137,7 @@ void predict(mxArray *plhs[], const mxArray *prhs[], struct svm_model *model, co
 			if(mexCallMATLAB(1, lhs, 1, rhs, "full"))
 			{
 				mexPrintf("Error: cannot full testing instance matrix\n");
-				fake_answer(plhs);
+				fake_answer(nlhs, plhs);
 				return;
 			}
 			ptr_instance = mxGetPr(lhs[0]);
@@ -139,7 +150,7 @@ void predict(mxArray *plhs[], const mxArray *prhs[], struct svm_model *model, co
 			if(mexCallMATLAB(1, pplhs, 1, pprhs, "transpose"))
 			{
 				mexPrintf("Error: cannot transpose testing instance matrix\n");
-				fake_answer(plhs);
+				fake_answer(nlhs, plhs);
 				return;
 			}
 		}
@@ -148,19 +159,19 @@ void predict(mxArray *plhs[], const mxArray *prhs[], struct svm_model *model, co
 	if(predict_probability)
 	{
 		if(svm_type==NU_SVR || svm_type==EPSILON_SVR)
-			mexPrintf("Prob. model for test data: target value = predicted value + z,\nz: Laplace distribution e^(-|z|/sigma)/(2sigma),sigma=%g\n",svm_get_svr_probability(model));
+			info("Prob. model for test data: target value = predicted value + z,\nz: Laplace distribution e^(-|z|/sigma)/(2sigma),sigma=%g\n",svm_get_svr_probability(model));
 		else
 			prob_estimates = (double *) malloc(nr_class*sizeof(double));
 	}
 
-	plhs[0] = mxCreateDoubleMatrix(testing_instance_number, 1, mxREAL);
+	tplhs[0] = mxCreateDoubleMatrix(testing_instance_number, 1, mxREAL);
 	if(predict_probability)
 	{
 		// prob estimates are in plhs[2]
 		if(svm_type==C_SVC || svm_type==NU_SVC)
-			plhs[2] = mxCreateDoubleMatrix(testing_instance_number, nr_class, mxREAL);
+			tplhs[2] = mxCreateDoubleMatrix(testing_instance_number, nr_class, mxREAL);
 		else
-			plhs[2] = mxCreateDoubleMatrix(0, 0, mxREAL);
+			tplhs[2] = mxCreateDoubleMatrix(0, 0, mxREAL);
 	}
 	else
 	{
@@ -169,14 +180,14 @@ void predict(mxArray *plhs[], const mxArray *prhs[], struct svm_model *model, co
 		   svm_type == EPSILON_SVR ||
 		   svm_type == NU_SVR ||
 		   nr_class == 1) // if only one class in training data, decision values are still returned.
-			plhs[2] = mxCreateDoubleMatrix(testing_instance_number, 1, mxREAL);
+			tplhs[2] = mxCreateDoubleMatrix(testing_instance_number, 1, mxREAL);
 		else
-			plhs[2] = mxCreateDoubleMatrix(testing_instance_number, nr_class*(nr_class-1)/2, mxREAL);
+			tplhs[2] = mxCreateDoubleMatrix(testing_instance_number, nr_class*(nr_class-1)/2, mxREAL);
 	}
 
-	ptr_predict_label = mxGetPr(plhs[0]);
-	ptr_prob_estimates = mxGetPr(plhs[2]);
-	ptr_dec_values = mxGetPr(plhs[2]);
+	ptr_predict_label = mxGetPr(tplhs[0]);
+	ptr_prob_estimates = mxGetPr(tplhs[2]);
+	ptr_dec_values = mxGetPr(tplhs[2]);
 	x = (struct svm_node*)malloc((feature_number+1)*sizeof(struct svm_node) );
 	for(instance_index=0;instance_index<testing_instance_number;instance_index++)
 	{
@@ -224,7 +235,7 @@ void predict(mxArray *plhs[], const mxArray *prhs[], struct svm_model *model, co
 			{
 				double *dec_values = (double *) malloc(sizeof(double) * nr_class*(nr_class-1)/2);
 				predict_label = svm_predict_values(model, x, dec_values);
-				if(nr_class == 1) 
+				if(nr_class == 1)
 					ptr_dec_values[instance_index] = 1;
 				else
 					for(i=0;i<(nr_class*(nr_class-1))/2;i++)
@@ -244,25 +255,21 @@ void predict(mxArray *plhs[], const mxArray *prhs[], struct svm_model *model, co
 		sumpt += predict_label*target_label;
 		++total;
 	}
-
-#if 0
-	// do not print debug messages	
 	if(svm_type==NU_SVR || svm_type==EPSILON_SVR)
 	{
-		mexPrintf("Mean squared error = %g (regression)\n",error/total);
-		mexPrintf("Squared correlation coefficient = %g (regression)\n",
+		info("Mean squared error = %g (regression)\n",error/total);
+		info("Squared correlation coefficient = %g (regression)\n",
 			((total*sumpt-sump*sumt)*(total*sumpt-sump*sumt))/
 			((total*sumpp-sump*sump)*(total*sumtt-sumt*sumt))
 			);
 	}
 	else
-		mexPrintf("Accuracy = %g%% (%d/%d) (classification)\n",
+		info("Accuracy = %g%% (%d/%d) (classification)\n",
 			(double)correct/total*100,correct,total);
-#endif
 
 	// return accuracy, mean squared error, squared correlation coefficient
-	plhs[1] = mxCreateDoubleMatrix(3, 1, mxREAL);
-	ptr = mxGetPr(plhs[1]);
+	tplhs[1] = mxCreateDoubleMatrix(3, 1, mxREAL);
+	ptr = mxGetPr(tplhs[1]);
 	ptr[0] = (double)correct/total*100;
 	ptr[1] = error/total;
 	ptr[2] = ((total*sumpt-sump*sumt)*(total*sumpt-sump*sumt))/
@@ -271,16 +278,28 @@ void predict(mxArray *plhs[], const mxArray *prhs[], struct svm_model *model, co
 	free(x);
 	if(prob_estimates != NULL)
 		free(prob_estimates);
+
+	switch(nlhs)
+	{
+		case 3:
+			plhs[2] = tplhs[2];
+			plhs[1] = tplhs[1];
+		case 1:
+		case 0:
+			plhs[0] = tplhs[0];
+	}
 }
 
 void exit_with_help()
 {
 	mexPrintf(
-		"Usage: [predicted_label, accuracy, decision_values/prob_estimates] = svmpredict_mex(testing_label_vector, testing_instance_matrix, model, 'libsvm_options')\n"
+		"Usage: [predicted_label, accuracy, decision_values/prob_estimates] = svmpredict(testing_label_vector, testing_instance_matrix, model, 'libsvm_options')\n"
+		"       [predicted_label] = svmpredict(testing_label_vector, testing_instance_matrix, model, 'libsvm_options')\n"
 		"Parameters:\n"
 		"  model: SVM model structure from svmtrain.\n"
 		"  libsvm_options:\n"
 		"    -b probability_estimates: whether to predict probability estimates, 0 or 1 (default 0); one-class SVM not supported yet\n"
+		"    -q : quiet mode (no outputs)\n"
 		"Returns:\n"
 		"  predicted_label: SVM prediction output vector.\n"
 		"  accuracy: a vector with accuracy, mean squared error, squared correlation coefficient.\n"
@@ -293,17 +312,18 @@ void mexFunction( int nlhs, mxArray *plhs[],
 {
 	int prob_estimate_flag = 0;
 	struct svm_model *model;
+	info = &mexPrintf;
 
-	if(nrhs > 4 || nrhs < 3)
+	if(nlhs == 2 || nlhs > 3 || nrhs > 4 || nrhs < 3)
 	{
 		exit_with_help();
-		fake_answer(plhs);
+		fake_answer(nlhs, plhs);
 		return;
 	}
 
 	if(!mxIsDouble(prhs[0]) || !mxIsDouble(prhs[1])) {
 		mexPrintf("Error: label vector and instance matrix must be double\n");
-		fake_answer(plhs);
+		fake_answer(nlhs, plhs);
 		return;
 	}
 
@@ -326,10 +346,10 @@ void mexFunction( int nlhs, mxArray *plhs[],
 			for(i=1;i<argc;i++)
 			{
 				if(argv[i][0] != '-') break;
-				if(++i>=argc)
+				if((++i>=argc) && argv[i-1][1] != 'q')
 				{
 					exit_with_help();
-					fake_answer(plhs);
+					fake_answer(nlhs, plhs);
 					return;
 				}
 				switch(argv[i-1][1])
@@ -337,10 +357,14 @@ void mexFunction( int nlhs, mxArray *plhs[],
 					case 'b':
 						prob_estimate_flag = atoi(argv[i]);
 						break;
+					case 'q':
+						i--;
+						info = &print_null;
+						break;
 					default:
 						mexPrintf("Unknown option: -%c\n", argv[i-1][1]);
 						exit_with_help();
-						fake_answer(plhs);
+						fake_answer(nlhs, plhs);
 						return;
 				}
 			}
@@ -350,7 +374,7 @@ void mexFunction( int nlhs, mxArray *plhs[],
 		if (model == NULL)
 		{
 			mexPrintf("Error: can't read model: %s\n", error_msg);
-			fake_answer(plhs);
+			fake_answer(nlhs, plhs);
 			return;
 		}
 
@@ -359,7 +383,7 @@ void mexFunction( int nlhs, mxArray *plhs[],
 			if(svm_check_probability_model(model)==0)
 			{
 				mexPrintf("Model does not support probabiliy estimates\n");
-				fake_answer(plhs);
+				fake_answer(nlhs, plhs);
 				svm_free_and_destroy_model(&model);
 				return;
 			}
@@ -367,17 +391,17 @@ void mexFunction( int nlhs, mxArray *plhs[],
 		else
 		{
 			if(svm_check_probability_model(model)!=0)
-				mexPrintf("Model supports probability estimates, but disabled in predicton.\n");
+				info("Model supports probability estimates, but disabled in predicton.\n");
 		}
 
-		predict(plhs, prhs, model, prob_estimate_flag);
+		predict(nlhs, plhs, prhs, model, prob_estimate_flag);
 		// destroy model
 		svm_free_and_destroy_model(&model);
 	}
 	else
 	{
 		mexPrintf("model file should be a struct array\n");
-		fake_answer(plhs);
+		fake_answer(nlhs, plhs);
 	}
 
 	return;
