@@ -1031,15 +1031,16 @@ if present.
 
 double xpt2d(uint64_t x) {
 	// x is little-endian 64bit IBM floating point format
+#if __BYTE_ORDER == __BIG_ENDIAN
+
+	mexErrMsgTxt("IEEE-to-IBM conversion on big-endian platform not supported, yet");
+	return(NaN);
+
+#elif __BYTE_ORDER==__LITTLE_ENDIAN
 	char c = *((char*)&x+7) & 0x7f;
 	uint64_t u = x;
 	*((char*)&u+7)=0;
 
-#if __BYTE_ORDER == __BIG_ENDIAN
-
-	mexErrMsgTxt("IEEE-to-IBM conversion on big-endian platform not supported, yet");
-
-#elif __BYTE_ORDER==__LITTLE_ENDIAN
 
 #if DEBUG
 	mexPrintf("xpt2d(%016Lx): [0x%x]\n",x,c);
@@ -1071,15 +1072,15 @@ double xpt2d(uint64_t x) {
 */
 
 uint64_t d2xpt(double x) {
-	uint64_t s,m;
-	int e;
-
 #if __BYTE_ORDER == __BIG_ENDIAN
 
 	mexErrMsgTxt("IEEE-to-IBM conversion on big-endian platform not supported, yet");
-
+	return(0x2eLL << 56);	// NaN - not a number
 
 #elif __BYTE_ORDER==__LITTLE_ENDIAN
+	uint64_t s,m;
+	int e;
+
 
 	if (x != x) return(0x2eLL << 56);	// NaN - not a number
 
